// Generated by CoffeeScript 1.10.0
(function() {
  var KicadGenerator, QedaElement, QedaLibrary, fs, mkdirp, path, request;

  fs = require('fs');

  mkdirp = require('mkdirp');

  path = require('path');

  request = require('sync-request');

  KicadGenerator = require('./kicad-generator');

  QedaElement = require('./qeda-element');

  QedaLibrary = (function() {
    function QedaLibrary(settings) {
      if (settings == null) {
        settings = {};
      }
      this.elementStyle = 'default';
      this.symbolStyle = 'default';
      this.patternStyle = 'default';
      this.symbol = {
        units: 'mm',
        gridSize: 5,
        textSize: 1
      };
      this.mergeObjects(this, settings);
      this.symbolDefs = [
        {
          regexp: /DIL(\d+)/,
          handler: 'dil'
        }
      ];
      this.patternDefs = [
        {
          regexp: /SOP(\d+)P(\d+)X(\d+)-(\d+)/,
          handler: 'sm/soic'
        }, {
          regexp: /SOIC(\d+)P(\d+)X(\d+)-(\d+)/,
          handler: 'sm/soic'
        }
      ];
      this.elements = [];
    }

    QedaLibrary.prototype.add = function(element) {
      var def, newElement;
      def = this.load(element);
      if (def.abstract) {
        console.error("'" + element + "': Cannot add abstract component, use it only as base for others");
        process.exit(1);
      }
      newElement = new QedaElement(this, def);
      this.elements.push(newElement);
      return newElement;
    };

    QedaLibrary.prototype.addPatternDefinition = function(regexp, handler) {
      return this.patternDefs.push({
        regexp: regexp,
        handler: handler
      });
    };

    QedaLibrary.prototype.calculate = function() {
      var e, i, len, ref;
      if (this._calculated == null) {
        this._calculated = false;
      }
      if (this._calculated) {
        return;
      }
      this.symbol.textSize = this.symbol.textSize * this.symbol.gridSize;
      ref = this.elements;
      for (i = 0, len = ref.length; i < len; i++) {
        e = ref[i];
        e.symbol.calculate(this.symbol.gridSize);
      }
      return this._calculated = true;
    };

    QedaLibrary.prototype.generateKicad = function(name) {
      var kicad;
      this.calculate();
      kicad = new KicadGenerator(this);
      return kicad.generate(name);
    };

    QedaLibrary.prototype.load = function(element) {
      var baseDef, baseElement, def, elementJson, localFile, res;
      elementJson = element.toLowerCase() + '.json';
      localFile = './library/' + elementJson;
      if (!fs.existsSync(localFile)) {
        res = request('GET', 'https://raw.githubusercontent.com/qeda/library/master/' + elementJson, {
          timeout: 3000
        });
        if (res.statusCode === 200) {
          mkdirp.sync(path.dirname(localFile));
          fs.writeFileSync(localFile, res.body);
          console.log("Loading '" + element + "': OK");
        } else {
          console.error("Loading '" + element + "': Error (" + res.statusCode + ")");
          process.exit(1);
        }
      }
      def = JSON.parse(fs.readFileSync(localFile));
      if (def.base != null) {
        baseElement = def.base;
        delete def.base;
        if (path.dirname(baseElement) === '.') {
          baseElement = path.dirname(element) + '/' + baseElement;
        }
        baseDef = this.load(baseElement);
        if (baseDef.abstract) {
          delete baseDef.abstract;
        }
        this.mergeObjects(baseDef, def);
        def = baseDef;
      }
      return def;
    };

    QedaLibrary.prototype.mergeObjects = function(dest, src) {
      var k, results, v;
      results = [];
      for (k in src) {
        v = src[k];
        if (typeof v === 'object' && dest.hasOwnProperty(k)) {
          results.push(this.mergeObjects(dest[k], v));
        } else {
          results.push(dest[k] = v);
        }
      }
      return results;
    };

    QedaLibrary.prototype.setElementStyle = function(style) {
      style = style.toLowerCase();
      this.elementStyle = style;
      if (this.symbolStyle === 'default' || style === 'default') {
        this.symbolStyle = style;
      }
      if (this.patternStyle === 'default' || style === 'default') {
        return this.patternStyle = style;
      }
    };

    QedaLibrary.prototype.setPatternStyle = function(style) {
      return this.patternStyle = style.toLowerCase();
    };

    QedaLibrary.prototype.setSymbolStyle = function(style) {
      return this.symbolStyle = style.toLowerCase();
    };

    return QedaLibrary;

  })();

  module.exports = QedaLibrary;

}).call(this);
